--[[ Copyright (c) 2020 Picklock
 * Part of Picklock's rocket silo stats
 * control.lua
 * Version 1.0.0.10
 *
 * See LICENSE.MD in the project directory for license information.
--]]

--variables
	--general
	local PI_debug = false
	local PI_print = false --set to false in multiplayer to avoid desyncs
	local PI_log = false
		
	local PI_general ={
		name = "Picklock's rocket silo stats",
		version = "1.0.0"
	}

	--gui
	local pi_rss_mod_gui = require('mod-gui')
	local pi_rss_get_button_flow = pi_rss_mod_gui.get_button_flow
	local pi_rss_get_frame_flow = pi_rss_mod_gui.get_frame_flow
	local pi_rss_button_style = pi_rss_mod_gui.button_style

	--rss specific
	local pi_rss_col_count = 3
	local pi_rss_show_heading = true
	local pi_rss_show_stats = "Rocket launches and payload"
	local pi_rss_rocket_count = 0
	local pi_rss_show_average = false
	local pi_rss_average_per = "1 hour"
	local pi_rss_tick_first = 324000
	local pi_rss_tick_last = 324000
	local pi_rss_average_div = 216000
	local pi_rss_average_heading = "1 hour"
	local pi_rss_loc_item_name = ""
	local pi_rss_is_init = false

--functions
	--debugging information
	function print_debug(msg)
		if PI_debug then
			if PI_print then
				for _, myPlayer in pairs(game.players) do
					myPlayer.print("["..PI_general.name.." V" ..PI_general.version.. "]: "..msg)
				end
			end
			--if PI_log and PI_general.isAdmin then log("["..PI_general.name.." V" ..PI_general.version.. "]: "..msg) end
			if PI_log then log("["..PI_general.name.." V" ..PI_general.version.. "]: "..msg) end
		end
	end

	--calculate average values
	local function pi_rss_calc_average (myValue)
		local myDivisor = 1
		local pi_rss_time = pi_rss_tick_last - pi_rss_tick_first
		--if PI_debug then print_debug(" pi_rss_calc_average: myValue:"..tostring(myValue)) end
		--if PI_debug then print_debug(" pi_rss_calc_average: first tick:"..tostring(pi_rss_tick_first)) end
		--if PI_debug then print_debug(" pi_rss_calc_average: last tick:"..tostring(pi_rss_tick_last)) end
		--if PI_debug then print_debug(" pi_rss_calc_average: pi_rss_time:"..tostring(pi_rss_time)) end
		if pi_rss_time == 0 then pi_rss_time = 1 end
		if pi_rss_time > 18000 then myDivisor = pi_rss_time / pi_rss_average_div end
		--if pi_rss_time > pi_rss_average_div then myDivisor = pi_rss_time / pi_rss_average_div end
		local myResult = myValue / myDivisor
		--if PI_debug then print_debug(" pi_rss_calc_average: myResult:"..tostring(myResult)) end
		return math.floor(myResult*100)/100
	end
	
	--generate and show rss gui
	local function pi_rss_rebuild(myPlayer)
		--if PI_debug then print_debug("function started: pi_rss_rebuild") end
		local pi_rss_gui_left = pi_rss_get_frame_flow(myPlayer)
		local pi_rss_frame = pi_rss_gui_left['pi_rss_rocket-silo-stats']
		local pi_rss_visible = true
		local pi_rss_rocket_stats = nil
		local pi_rss_average_value = 0
		if pi_rss_frame then
			pi_rss_visible = pi_rss_frame.visible
			pi_rss_rocket_stats = pi_rss_frame.children[1].children[1]
			pi_rss_rocket_stats.clear()
		else
			pi_rss_frame = pi_rss_gui_left.add{
				type = 'frame',
				name = 'pi_rss_rocket-silo-stats',
				caption = {'pi_rss_rocket-silo-stats.pi_rss_title'},
				style = 'pi_rss_frame',
			}
			pi_rss_rocket_stats = pi_rss_frame.add{
				type = 'frame',
				style = 'inside_shallow_frame',
			}.add{
				type = 'table',
				style = 'bordered_table',
				column_count = pi_rss_col_count,
			}
			pi_rss_rocket_stats.style.column_alignments[3] = "right"
			if pi_rss_col_count == 4 then pi_rss_rocket_stats.style.column_alignments[4] = "right" end
		end
		-- add column names
		if pi_rss_show_heading then
			pi_rss_rocket_stats.add{type = 'label', caption = {''}, style = 'caption_label'}
			pi_rss_rocket_stats.add{type = 'label', caption = {'pi_rss_rocket-silo-stats.pi_rss_name'}, style = 'caption_label'}
			pi_rss_rocket_stats.add{type = 'label', tags = {item = 'pi-rss-heading'}, caption = {'pi_rss_rocket-silo-stats.pi_rss_number'}, style = 'caption_label'}
			if pi_rss_show_average == true then
				local myHeading = ({"", {"pi_rss_rocket-silo-stats.pi_rss_average"}, " - ", pi_rss_average_heading})
				pi_rss_rocket_stats.add{type = 'label', tags = {item = 'pi-rss-heading'}, caption = myHeading, style = 'caption_label'}
			end
		end
		-- add rockets launched
		if pi_rss_show_stats == "Rocket launches and payload" or pi_rss_show_stats == "Rocket launches only" then
			--if PI_debug then print_debug("pi_rss_rebuild: pi_rss_rocket_count = " ..tostring(pi_rss_rocket_count)) end
			pi_rss_rocket_stats.add{type = 'sprite', style = 'small_text_image', sprite = 'item/pi-space-rocket'}
			pi_rss_rocket_stats.add{type = 'label', caption = {'item-name.rocket'}, style = 'bold_label'}
			pi_rss_rocket_stats.add{type = 'label', tags = {item = 'pi-space-rocket'}, caption = tostring(pi_rss_rocket_count)}
			if pi_rss_show_average == true then
				pi_rss_average_value = pi_rss_calc_average (pi_rss_rocket_count)
				pi_rss_rocket_stats.add{type = 'label', tags = {item = 'pi-space-rocket'}, caption = tostring(string.format ("%.2f", pi_rss_average_value))}
			end
		end
		-- add items
		if pi_rss_show_stats == "Rocket launches and payload" or pi_rss_show_stats == "Payload only" then
			local pi_rss_filters = global.pi_rss_filters
			local pi_rss_restricted = global.pi_rss_restricted
			for myItem, myCount in pairs(myPlayer.force.items_launched) do
				if pi_rss_filters[myItem] == pi_rss_restricted then
					pi_rss_loc_item_name = game.item_prototypes[myItem].localised_name
					pi_rss_rocket_stats.add{type = 'sprite', style = 'small_text_image', sprite = 'item/'..myItem}
					pi_rss_rocket_stats.add{type = 'label', caption = pi_rss_loc_item_name, style = 'bold_label'}
					pi_rss_rocket_stats.add{type = 'label', tags = {item = myItem}, caption = tostring(myCount)}
					if pi_rss_show_average == true then
						pi_rss_average_value = pi_rss_calc_average (myCount)
						pi_rss_rocket_stats.add{type = 'label', tags = {item = myItem}, caption = tostring(string.format ("%.2f", pi_rss_average_value))}
					end
				end
			end
		end
		if myPlayer.mod_settings['pi-rss-hide-empty'].value then
			if pi_rss_rocket_count == 0 then
				pi_rss_visible = false
			else
				pi_rss_visible = true
			end
		end
		pi_rss_frame.visible = pi_rss_visible
	end

	--update and show rss gui
	local function pi_rss_update(myPlayer, myPayload)
		--if PI_debug then print_debug("function started: pi_rss_update") end
		local pi_rss_gui_left = pi_rss_get_frame_flow(myPlayer)
		local pi_rss_frame = pi_rss_gui_left['pi_rss_rocket-silo-stats']
		local pi_rss_rocket_stats = pi_rss_frame.children[1].children[1]
		local pi_rss_cells = pi_rss_rocket_stats.children
		local pi_rss_added = false
		local pi_rss_get_item_launched = myPlayer.force.get_item_launched
		local pi_rss_found = {}
		for i = pi_rss_col_count, #pi_rss_cells, pi_rss_col_count do
			local pi_rss_el = pi_rss_cells[i]
			local myItem = pi_rss_el.tags.item
			-- update rocket launched
			if myItem == "pi-space-rocket" then
				if pi_rss_show_average == true then
					j = i - 1
					pi_rss_average_value = pi_rss_calc_average (pi_rss_rocket_count)
					pi_rss_el.caption = tostring(string.format ("%.2f", pi_rss_average_value))
				else 
					j = i
				end
				local pi_rss_el2 = pi_rss_cells[j]
				pi_rss_el2.caption = tostring(pi_rss_rocket_count)
			end
			-- update payload
			if myPayload[myItem] then
				pi_rss_found[myItem] = true
				if pi_rss_show_average == true then
					j = i - 1
					pi_rss_average_value = pi_rss_calc_average (pi_rss_get_item_launched(myItem))
					pi_rss_el.caption = tostring(string.format ("%.2f", pi_rss_average_value))
				else 
					j = i
				end
				local pi_rss_el2 = pi_rss_cells[j]
				pi_rss_el2.caption = tostring(pi_rss_get_item_launched(myItem))
			else
				-- update average non payload
				if pi_rss_show_average == true and myItem ~= "pi-space-rocket" and myItem ~= "pi-rss-heading" then
					local pi_rss_el2 = pi_rss_cells[i-1]
					local intTemp = tonumber(pi_rss_el2.caption)
					pi_rss_average_value = pi_rss_calc_average (intTemp)
					pi_rss_el.caption = tostring(string.format ("%.2f", pi_rss_average_value))
				end
			end
		end
		-- add items
		if pi_rss_show_stats == "Rocket launches and payload" or pi_rss_show_stats == "Payload only" then
			local pi_rss_filters = global.pi_rss_filters
			local pi_rss_restricted = global.pi_rss_restricted
			for myItem, myCount in pairs(myPayload) do
				if not pi_rss_found[myItem] and pi_rss_filters[myItem] == pi_rss_restricted then
					pi_rss_added = true
					pi_rss_loc_item_name = game.item_prototypes[myItem].localised_name
					pi_rss_rocket_stats.add{type = 'sprite', style = 'small_text_image', sprite = 'item/'..myItem}
					pi_rss_rocket_stats.add{type = 'label', caption = pi_rss_loc_item_name, style = 'bold_label'}
					pi_rss_rocket_stats.add{type = 'label', tags = {item = myItem}, caption = tostring(myCount)}
					if pi_rss_show_average == true then
						pi_rss_average_value = pi_rss_calc_average (myCount)
						pi_rss_rocket_stats.add{type = 'label', tags = {item = myItem}, caption = tostring(string.format ("%.2f", pi_rss_average_value))}
					end
				end
			end
		end
		local pi_rss_player_settings = myPlayer.mod_settings
		if pi_rss_added and pi_rss_player_settings['pi-rss-hide-empty'].value then
			pi_rss_frame.visible = true
			pi_rss_get_button_flow(myPlayer)['pi_rss_but_toggle'].visible =
				pi_rss_player_settings['pi-rss-show-tb'].value
		end
	end
	
	--destroy gui
	local function pi_rss_destroy(myPlayer)
		--if PI_debug then print_debug("function started: pi_rss_destroy") end
		local pi_rss_gui_left = pi_rss_get_frame_flow(myPlayer)
		local pi_rss_frame = pi_rss_gui_left['pi_rss_rocket-silo-stats']
		local pi_rss_visible = true
		local pi_rss_was_empty = true
		local pi_rss_rocket_stats = nil
		local pi_rss_average_value = 0
		if pi_rss_frame then
			--if PI_debug then print_debug("pi_rss_destroy: pi_rss_frame to destroy") end
			pi_rss_frame.destroy ()
		end
	end
	
	--set divisor
	local function pi_rss_get_divisor ()
		-- 1 minute
		if pi_rss_average_per =="1 minute" then 
			pi_rss_average_div = 3600 
			pi_rss_average_heading = ({"", {"string-mod-setting.pi-rss-average-per-1 minute"}})
		end
		-- 10 minutes
		if pi_rss_average_per =="10 minutes" then 
			pi_rss_average_div = 36000 
			pi_rss_average_heading = ({"", {"string-mod-setting.pi-rss-average-per-10 minutes"}})
		end
		-- 30 minutes
		if pi_rss_average_per =="30 minutes" then 
			pi_rss_average_div = 108000 
			pi_rss_average_heading = ({"", {"string-mod-setting.pi-rss-average-per-30 minutes"}})
		end
		-- 1 hour
		if pi_rss_average_per =="1 hour" then 
			pi_rss_average_div = 216000 
			pi_rss_average_heading = ({"", {"string-mod-setting.pi-rss-average-per-1 hour"}})
		end
		-- 5 hours
		if pi_rss_average_per =="5 hours" then 
			pi_rss_average_div = 1080000 
			pi_rss_average_heading = ({"", {"string-mod-setting.pi-rss-average-per-5 hours"}})
		end
		-- 10 hours
		if pi_rss_average_per =="10 hours" then 
			pi_rss_average_div = 2160000 
			pi_rss_average_heading = ({"", {"string-mod-setting.pi-rss-average-per-10 hours"}})
		end
	end
	
	--filter update
	local function pi_rss_update_filters(myWhitelist, myBlacklist)
		local pi_rss_filters = {}
		local pi_rss_restricted = nil
		--Whitelist
		for myItem in string.gmatch(myWhitelist, '([^,%s]+)') do
			pi_rss_restricted = true
			pi_rss_filters[myItem] = true
		end
		global.pi_rss_restricted = pi_rss_restricted
		--Blacklist
		for myItem in string.gmatch(myBlacklist, '([^,%s]+)') do
			pi_rss_filters[myItem] = false
		end
		--set filters
		global.pi_rss_filters = pi_rss_filters
	end
	
	--Prepare filters depending on pi-rss-track-items
	local function pi_rss_prepare_filters()
		local myWhitelist = ""
		local myBlacklist = ""
		global.pi_rss_filters={}
		global.pi_rss_restricted = nil
		local pi_rss_map_settings = settings.global
		local pi_rss_setting = pi_rss_map_settings['pi-rss-track-items'].value
		if pi_rss_setting=="all" then return end
		if pi_rss_setting=="satellite" then myWhitelist="satellite" end
		if pi_rss_setting=="satellite and fisch" then myWhitelist="satellite, raw-fish" end
		if pi_rss_setting=="filter-lists" then 
			myWhitelist=pi_rss_map_settings['pi-rss-whitelist'].value
			myBlacklist=pi_rss_map_settings['pi-rss-blacklist'].value
		end
		pi_rss_update_filters (myWhitelist, myBlacklist)
	end

	--toggle gui
	local function pi_rss_on_toggle(event)
		-- event.player_index   player who triggered the custom input (unlisted in API)
		local myPlayer = game.get_player(event.player_index)
		local pi_rss_frame = pi_rss_get_frame_flow(myPlayer)['pi_rss_rocket-silo-stats']
		if not myPlayer.mod_settings['pi-rss-hide-empty'].value or #pi_rss_frame.children[1].children[1].children > 0 then
			pi_rss_frame.visible = not pi_rss_frame.visible
		end
	end

	--gui clicked
	local function pi_rss_on_gui_clicked(event)
		if event.element.name == 'pi_rss_but_toggle' then
			pi_rss_on_toggle(event)
		end
	end

	--create and show toggle button
	local function pi_rss_init_but_toggle(myPlayer)
		local pi_rss_but_gui = pi_rss_get_button_flow(myPlayer)
		local pi_rss_toggle_element = pi_rss_but_gui['pi_rss_but_toggle']
		if not pi_rss_toggle_element then
			pi_rss_toggle_element = pi_rss_but_gui.add{
				type = 'sprite-button',
				name = 'pi_rss_but_toggle',
				sprite = 'item/pi-space-rocket',
				style = pi_rss_button_style,
			}
		end
		local pi_rss_player_settings = myPlayer.mod_settings
		pi_rss_toggle_element.visible = pi_rss_player_settings['pi-rss-show-tb'].value
			and (not pi_rss_player_settings['pi-rss-hide-empty'].value
			or pi_rss_rocket_count > 0)
	end
	
	--rocket count
	local function pi_rss_get_rockets_lauched()
		-- if using mod in old save then this function will count the launched rockets
		-- couts launched rockets from rocket-silos
		-- if there was a rocket-silo destroyed or removed in the past then this launched rockets could not be count
		
		--if PI_debug then print_debug("function started: pi_rss_get_rockets_lauched") end
		--filter built rocket-silos
		local pi_rss_db_rocket_silo = game.surfaces[1].find_entities_filtered{name = "rocket-silo"}
		--if PI_debug then print_debug("pi_rss_get_rockets_lauched: fount rocket-silos: "..tostring(#pi_rss_db_rocket_silo)) end

		--count launched rockets
		for _, myRocketSilo in pairs(pi_rss_db_rocket_silo) do
			pi_rss_rocket_count = pi_rss_rocket_count + myRocketSilo.products_finished
			--if PI_debug then print_debug("pi_rss_get_rockets_lauched: Rocket-Silo.unitnumber: "..myRocketSilo.unit_number.." Rocket-Silo.status: "..myRocketSilo.status) end
			--if PI_debug then print_debug("pi_rss_get_rockets_lauched: Rocket-Silo.status: "..myRocketSilo.status) end
			if myRocketSilo.status == defines.entity_status.waiting_to_launch_rocket then pi_rss_rocket_count = pi_rss_rocket_count - 1 end
		end

		--check laucht rockets by payload
		if game.tick > 1 then
			local myPlayer = game.get_player(1)
			local myCountSum = 0
			for myItem, myCount in pairs(myPlayer.force.items_launched) do
				myCountSum = myCountSum + myCount
			end
			if myCountSum == 0 then pi_rss_rocket_count = 0 end
		end
		
		if pi_rss_rocket_count < 0 then  pi_rss_rocket_count = 0 end
		global.pi_rss_rocket_count = pi_rss_rocket_count
	end
	
	--init gui
	local function pi_rss_init_gui()
		--if PI_debug then print_debug("function started: pi_rss_init_gui") end
		
		-- load global settings
		pi_rss_rocket_count = global.pi_rss_rocket_count
		pi_rss_tick_first = global.pi_rss_tick_first
		pi_rss_tick_last = global.pi_rss_tick_last
		
		local pi_rss_map_settings = settings.global
		-- set pi-rss-show-heading from mod settings
		pi_rss_show_heading = pi_rss_map_settings['pi-rss-show-heading'].value
		-- set pi-rss-show-in-stats from mod_settings
		pi_rss_show_stats = pi_rss_map_settings['pi-rss-show-in-stats'].value		
		-- set pi-rss-show-average from mod_settings
		pi_rss_show_average = pi_rss_map_settings['pi-rss-show-average'].value		
		if pi_rss_show_average == true then 
			pi_rss_col_count = 4			
		else
			pi_rss_col_count = 3
		end
		-- set pi-rss-average-per from mod_settings
		pi_rss_average_per = pi_rss_map_settings['pi-rss-average-per'].value
		pi_rss_get_divisor()

		--Filter update
		pi_rss_prepare_filters()
		--show gui
		for _, player in pairs(game.connected_players) do
			pi_rss_rebuild(player)
			pi_rss_init_but_toggle(player)
		end
	end
	
	--rocket lauched
	local function pi_rss_on_rocket_launched(event)
		--if PI_debug then print_debug("function started: pi_rss_on_rocket_launched") end
		pi_rss_rocket_count = global.pi_rss_rocket_count
		pi_rss_is_init = global.pi_rss_is_init
		-- get tick
		local myTick = event.tick
		pi_rss_tick_last = myTick
		global.pi_rss_tick_last = pi_rss_tick_last
		if pi_rss_rocket_count == 0 then 
			pi_rss_tick_first = myTick
			global.pi_rss_tick_first = pi_rss_tick_first
		end
		--if PI_debug then print_debug("pi_rss_on_rocket_launched: first tick:"..tostring(pi_rss_tick_first)) end
		--if PI_debug then print_debug("pi_rss_on_rocket_launched: last tick:"..tostring(pi_rss_tick_last)) end
		local myTemp = global.pi_rss_tick_last
		--if PI_debug then print_debug("pi_rss_on_rocket_launched: global last tick:"..tostring(myTemp)) end
		
		-- update rocket counter
		pi_rss_rocket_count = pi_rss_rocket_count +1
		global.pi_rss_rocket_count = pi_rss_rocket_count
		--if PI_debug then print_debug("pi_rss_on_rocket_launched: rocket count:"..tostring(pi_rss_rocket_count)) end
		-- get rocket payload and update gui
		local pi_rss_rocket = event.rocket
		local pi_rss_payload = pi_rss_rocket.get_inventory(defines.inventory.rocket).get_contents()
		if pi_rss_is_init then
			-- init - update
			for _, myPlayer in pairs(pi_rss_rocket.force.connected_players) do
				pi_rss_update(myPlayer, pi_rss_payload)
			end
		else
			-- not jet init - init gui
			pi_rss_is_init = true
			global.pi_rss_is_init = pi_rss_is_init
			pi_rss_init_gui ()	
		end
	end
	
	--load mp settings
	local function pi_rss_load_mp_settings (event)
		--if PI_debug then print_debug("function started: pi_rss_load_mp_settings") end
		-- load global variables
		pi_rss_rocket_count = global.pi_rss_rocket_count		
		--if PI_debug then print_debug("pi_rss_load_mp_settings: rocket count: "..tostring(pi_rss_rocket_count)) end
		pi_rss_tick_first = global.pi_rss_tick_first
		--if PI_debug then print_debug("pi_rss_load_mp_settings: first tick: "..tostring(pi_rss_tick_first)) end
		pi_rss_tick_last = global.pi_rss_tick_last
		--if PI_debug then print_debug("pi_rss_load_mp_settings: last tick: "..tostring(pi_rss_tick_last)) end
		pi_rss_is_init = global.pi_rss_is_init
		--if PI_debug then print_debug("pi_rss_load_mp_settings: pi_rss_is_init: "..tostring(pi_rss_is_init)) end
		-- load map settings
		local myPlayer = game.get_player(event.player_index)
		local pi_rss_map_settings = settings.global
		pi_rss_show_heading = pi_rss_map_settings['pi-rss-show-heading'].value
		--if PI_debug then print_debug("pi_rss_load_mp_settings: pi_rss_show_heading: "..tostring(pi_rss_show_heading)) end
		pi_rss_show_stats = pi_rss_map_settings['pi-rss-show-in-stats'].value
		--if PI_debug then print_debug("pi_rss_load_mp_settings: pi_rss_show_stats: "..pi_rss_show_stats) end

		local myWhitelist = pi_rss_map_settings['pi-rss-whitelist'].value
		--if PI_debug then print_debug("pi_rss_load_mp_settings: Whitelist: "..myWhitelist) end
		local myBlacklist = pi_rss_map_settings['pi-rss-blacklist'].value
		--if PI_debug then print_debug("pi_rss_load_mp_settings: Blacklist: "..myBlacklist) end

		pi_rss_show_average = pi_rss_map_settings['pi-rss-show-average'].value		
		--if PI_debug then print_debug("pi_rss_load_mp_settings: pi_rss_show_average: "..tostring(pi_rss_show_average)) end
		if pi_rss_show_average == true then 
			pi_rss_col_count = 4			
		else
			pi_rss_col_count = 3
		end
		pi_rss_average_per = pi_rss_map_settings['pi-rss-average-per'].value
		--if PI_debug then print_debug("pi_rss_load_mp_settings: pi_rss_average_per: "..pi_rss_average_per) end
		pi_rss_get_divisor ()
		pi_rss_destroy (myPlayer)
	end

	--mod settings changed
	local function pi_rss_on_settings_changed(event)
		local mySetting = event.setting
		local myPlayer = game.get_player(event.player_index)
		local pi_rss_map_settings = settings.global
		if mySetting == 'pi-rss-whitelist' or mySetting == 'pi-rss-blacklist' or mySetting == 'pi-rss-track-items' then
			pi_rss_init_gui()
		elseif mySetting == 'pi-rss-show-heading' then
			pi_rss_show_heading = pi_rss_map_settings['pi-rss-show-heading'].value
			pi_rss_init_gui()			
		elseif mySetting == 'pi-rss-show-in-stats' then
			pi_rss_show_stats = pi_rss_map_settings['pi-rss-show-in-stats'].value
			pi_rss_init_gui()			
		elseif mySetting == 'pi-rss-show-average' then
			pi_rss_show_average = pi_rss_map_settings['pi-rss-show-average'].value		
			if pi_rss_show_average == true then 
				pi_rss_col_count = 4			
			else
				pi_rss_col_count = 3
			end
			pi_rss_destroy (myPlayer)
			pi_rss_init_gui()			
		elseif mySetting == 'pi-rss-average-per' then
			pi_rss_average_per = pi_rss_map_settings['pi-rss-average-per'].value
			pi_rss_get_divisor ()
			if pi_rss_show_average == true then pi_rss_init_gui() end
		elseif mySetting == 'pi-rss-show-tb' then
			pi_rss_get_button_flow(myPlayer)['pi_rss_but_toggle'].visible = myPlayer.mod_settings['pi-rss-show-tb'].value
		elseif mySetting == 'pi-rss-hide-empty' then
			local pi_rss_frame = pi_rss_get_frame_flow(myPlayer)['pi_rss_rocket-silo-stats']
			local my_player_settings = myPlayer.mod_settings
			if not my_player_settings['pi-rss-hide-empty'].value then
				-- if false, show button = true
				pi_rss_get_button_flow(myPlayer)['pi_rss_but_toggle'].visible = my_player_settings['pi-rss-show-tb'].value
			elseif pi_rss_rocket_count == 0 then
				-- if true and no stats -> visible = false
				pi_rss_frame.visible = false
				pi_rss_get_button_flow(myPlayer)['pi_rss_but_toggle'].visible = false
			end
		end
	end

	--configuration changed
	local function pi_rss_on_config_changed(data)
		pi_rss_init_gui()
	end

	--player joined game
	local function pi_rss_on_player_joined_game(event)
		--if PI_debug then print_debug("function started: pi_rss_on_player_joined_game") end
		local myIndex = event.player_index
		--if PI_debug then print_debug("pi_rss_load_mp_settings: event.player_index: "..tostring(myIndex)) end
		if myIndex ~= 1 then 
			pi_rss_load_mp_settings(event) 
		else
			pi_rss_init_gui ()	
		end
		local myPlayer = game.get_player(event.player_index)
		-- show gui
		pi_rss_rebuild(myPlayer)
		pi_rss_init_but_toggle(myPlayer)
	end
	
	-- on_init
	local function pi_rss_on_init()

		-- Count launched rockets
		--if PI_debug then print_debug("function started: pi_rss_on_init") end
		if global.pi_rss_rocket_count == nil then
			--global variable not set
			--if PI_debug then print_debug("pi_rss_on_init: no global.pi_rss_rocket_count") end
			--get rocket count
			pi_rss_get_rockets_lauched ()
		else
			--global variable set - set local variable to global variable
			--if PI_debug then print_debug("pi_rss_on_init: global.pi_rss_rocket_count = " ..tostring(global.pi_rss_rocket_count)) end
			pi_rss_rocket_count = global.pi_rss_rocket_count
		end
		--if PI_debug then print_debug("pi_rss_on_init: pi_rss_rocket_count = " ..tostring(pi_rss_rocket_count)) end
		--if PI_debug then print_debug("pi_rss_on_init: global.pi_rss_rocket_count = " ..tostring(global.pi_rss_rocket_count)) end

		-- get tick from global
		if global.pi_rss_tick_last == nil then 
			pi_rss_tick_last = game.tick
		else
			pi_rss_tick_last = global.pi_rss_tick_last
			if pi_rss_tick_last < pi_rss_tick_last then pi_rss_tick_last = 324000 end
		end
		global.pi_rss_tick_first = pi_rss_tick_first
		global.pi_rss_tick_last = pi_rss_tick_last

		--if global.pi_rss_tick_first ~= nil then pi_rss_tick_first = global.pi_rss_tick_first end
		--if PI_debug then print_debug("pi_rss_on_init: first tick:"..tostring(pi_rss_tick_first)) end
		--if PI_debug then print_debug("pi_rss_on_init: last tick:"..tostring(pi_rss_tick_last)) end

		-- rebuild gui
		pi_rss_init_gui ()		
	end

--handler
	--rocket lauched
	script.on_event(defines.events.on_rocket_launched, pi_rss_on_rocket_launched)
	--gui clicked
	script.on_event(defines.events.on_gui_click, pi_rss_on_gui_clicked)
	--toggle key pressed
	script.on_event('pi-rss-toggle-key', pi_rss_on_toggle)
	--init
	script.on_init(pi_rss_on_init)
	--mod settings changed
	script.on_event(defines.events.on_runtime_mod_setting_changed, pi_rss_on_settings_changed)
	--configuration changed
	script.on_configuration_changed(pi_rss_on_config_changed)
	--player joined game
	script.on_event(defines.events.on_player_joined_game, pi_rss_on_player_joined_game)
